
/*
 * MGL graphics library
 * Copyright (C) 1999-2000, ing. Marek Gayer

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.

 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <string.h>

#include <mgl.h>

int mglShift;
int mglCtrl;
int mglAlt;

int mglMouseLeft;
int mglMouseRight;
int mglMouseMiddle;

static int mglMouseLeftPressTime;
static int mglMouseRightPressTime;
static int mglMouseMiddlePressTime;

float mglFps;

int mglMouseIn;
float mglMX;
float mglMY;
int mglMouseState;

int mglLastKey;

static int mglGetTime (void)
{
	return glutGet (GLUT_ELAPSED_TIME);
	// return time (NULL);
}

static mglMouseHandlerType mglMouseHandler;
static mglKeyboardHandlerType mglKeyboardHandler;

//mglTexture *mglMouseTexture;

int mglMouseCursor = GLUT_CURSOR_INHERIT;

int mglLastCursor;

void mglSetMouseCursor (int cursor)
{
  if (mglLastCursor != cursor)
  {
    if (mglLastCursor == GLUT_CURSOR_NONE)
      glutSetCursor (GLUT_CURSOR_INHERIT);
    mglLastCursor = cursor;
    mglMouseCursor = cursor;
    glutSetCursor (cursor);
  }
}

/*
void mglSetMouseTexture (mglTexture *tex)
{
  if (tex == NULL)
  {
    mglSetMouseCursor (GLUT_CURSOR_INHERIT);
  }
  mglMouseTexture = tex;
}

void mglSetMouseHandler (mglMouseHandlerType handler)
{
  mglMouseHandler = handler;
}
*/

extern int mglRegionX;
extern int mglRegionY;


static void mglUpdateMousePos (int x, int y)
{
	float newx, newy;
	
	if (x >= mglRegionX && x < (mglRegionX + mglWindowWidth))
	{
		if (y >= mglRegionY && y < (mglRegionY + mglWindowHeight))
		{
			newx = (float)x / mglWindowWidth;
			newy = ((float)mglWindowHeight - y) / mglWindowHeight; // uprava do dolniho rohu
			mglMouseIn = 1;
			x -= mglRegionX; y -= mglRegionY;

			if (fabs (newx - mglMX) > 0.000001 || fabs (newy - mglMY) > 0.000001)
				mglMouseMiddlePressTime = mglMouseLeftPressTime = mglMouseRightPressTime = 0;

			mglMX = newx;
			mglMY = newy;
			mglSetMouseCursor (mglMouseCursor);
			return;
		}
	}
	/*
	if (mglLastCursor != GLUT_CURSOR_INHERIT)
	{
	mglLastCursor = GLUT_CURSOR_INHERIT;
	glutSetCursor (GLUT_CURSOR_INHERIT);
	}
	*/
	mglMouseIn = 0;
}


static void mglGetModifiers (void)
{
  int m;

  m = glutGetModifiers ();
  mglShift = m & GLUT_ACTIVE_SHIFT;
  mglCtrl = m & GLUT_ACTIVE_CTRL;
  mglAlt = m & GLUT_ACTIVE_ALT;
}


static void mglMyMousePressFunc (int buttons, int state, int x, int y)
{
	int presstime;

	mglGetModifiers ();
  /*
  if (mglMouseTexture)
  {
    mglDrawTexture (mglMouseTexture, x, y);
  }
  */
	mglUpdateMousePos (x, y);

	presstime = mglGetTime ();

	if (state == GLUT_UP)
	{
		switch (buttons)
		{
			case GLUT_LEFT_BUTTON:
				mglMouseLeft = 0;
				state = MGL_LEFT_BUTTON_UP;
			break;
			case GLUT_RIGHT_BUTTON:
				mglMouseRight = 0;
				state = MGL_RIGHT_BUTTON_UP;
			break;
			case GLUT_MIDDLE_BUTTON:
				state = MGL_MIDDLE_BUTTON_UP;
				mglMouseMiddle = 0;
			break;
		}
	}
	else
	{
		switch (buttons)
		{
			case GLUT_LEFT_BUTTON:
				presstime = mglGetTime ();
				state = MGL_LEFT_BUTTON_DOWN;
				if (presstime - mglMouseLeftPressTime > 300) 
					mglMouseLeft = 1;
				else
					mglMouseLeft = 2;
				mglMouseLeftPressTime = presstime;
				mglMouseRightPressTime = mglMouseMiddlePressTime = 0;
				break;
			case GLUT_RIGHT_BUTTON:
				state = MGL_RIGHT_BUTTON_DOWN;
				mglMouseRight = 1;
				mglMouseRightPressTime = presstime;
				mglMouseMiddlePressTime = mglMouseLeftPressTime = 0;
				break;
			case GLUT_MIDDLE_BUTTON:
				state = MGL_MIDDLE_BUTTON_DOWN;
				mglMouseMiddle = 1;
				mglMouseMiddlePressTime = presstime;
				mglMouseLeftPressTime = mglMouseRightPressTime = 0;
				break;
		}
  }

  if ((mglMX == -1) && (mglMY == -1))
    mglMouseIn = 0;

  mglMouseState = state;

  if (mglMouseHandler)
    mglMouseHandler (state);
}

static void mglMyMouseMoveFunc (int x, int y)
{
//	mglGetModifiers ();

  mglUpdateMousePos (x, y);

  /*
  if (mglMouseRight) mglMouseRight++;
  if (mglMouseLeft) mglMouseLeft++;
  if (mglMouseMiddle) mglMouseMiddle++;
  */
}

static void mglMouseEntryFunc (int state)
{
  if (state == GLUT_ENTERED)
  {
    mglMouseIn = 1;
  }
  else
  {
    mglMX = -1;
    mglMY = -1;
    mglMouseIn = 0;
  }
}




void mglMyKeyboardFunc (unsigned char key, int x, int y)
{
  mglGetModifiers ();

  mglUpdateMousePos (x, y);

  mglLastKey = key;

  if (mglKeyboardHandler)
    mglKeyboardHandler (key);

  mglLastKey = 0;
}

void mglMySpecialFunc (int key, int x, int y)
{
  mglGetModifiers ();

  mglLastKey = 256+key;

  if (mglKeyboardHandler)
    mglKeyboardHandler (256 + key);

  mglLastKey = 0;
}

void mglSetKeyboardHandler (mglKeyboardHandlerType handler)
{
  mglKeyboardHandler = handler;
}

void mglSetMouseHandler (mglMouseHandlerType handler)
{
  mglMouseHandler = handler;
}

int mglTime;
int mglSystemTime;

void mglDelay (int ms)
{
	int time = mglTime;

	while (mglTime - time < ms)
		;
}

static mglOriginalTime;
static mglLastTime;
static mglFirstTime;

extern int mglLastFrames;

void mglTimer (void)
{
	int milisec;
	int mytime;

    mytime = mglGetTime ();

    if (mglLastFrames == 0)
    {
      mglLastTime = mytime;
    }

	mglSystemTime = mglFirstTime + mytime;
	mglTime = mytime - mglOriginalTime;

    milisec = mytime - mglLastTime;
    if (milisec > 999)
    {
//      mglFps = ((float)milisec * mglLastFrames) / 1000;
      mglFps = ((float)mglLastFrames * 1000) / milisec;
      mglLastFrames = 0;
      mglLastTime = mytime;
    }
}

void mglShowFPS (mglFont *font, float x, float y)
{
  char s [9];
  int slen;
  float csize;

  sprintf (s, "%iFPS", (int) mglFps);
	slen = strlen (s);
	csize = font->w * mglTextureZoomX;
  mglPuts (font, s, x - slen * csize, y);
}


void mglResetTimer (void)
{
	time_t systime = {0};
	struct tm *newtime;

	mglLastTime = mglGetTime ();
    newtime = localtime(&systime); /* Convert to local time. */
	
	if (!mglFirstTime)
		mglFirstTime = 1000 * ((60 * (60*newtime->tm_hour + newtime->tm_min)) + newtime->tm_sec) + mglLastTime % 1000;

	mglTime = 0;
	mglOriginalTime = mglLastTime;
}

void mglInitInput (void)
{
//  glutTimerFunc (100, mglTimer, MARKOVA_FREKVENCE);
  glutKeyboardFunc (mglMyKeyboardFunc);
  glutSpecialFunc (mglMySpecialFunc);
  glutMouseFunc (mglMyMousePressFunc);
  glutPassiveMotionFunc (mglMyMouseMoveFunc);
  glutMotionFunc (mglMyMouseMoveFunc);
  glutEntryFunc (mglMouseEntryFunc);
  
  mglFirstTime = time (NULL);
}


void mglTimerCall (void)
{
  mglTimer ();

  mglFrames++;
  mglLastFrames++;
}

void mglSleep (int ms)
{
	if (ms)
	{
	#ifdef _WIN32
		_sleep (ms);
	#else
		int origtime;

		origtime = mglGetTime ();
		
		while (mglGetTime () - origtime < ms)
			;
	#endif
	}
}
